"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const http_1 = require("@foxify/http");
const inject_1 = __importDefault(require("@foxify/inject"));
const router_1 = __importDefault(require("@foxify/router"));
const proxyAddr = __importStar(require("proxy-addr"));
const qs = __importStar(require("qs"));
const serve_static_1 = __importDefault(require("serve-static"));
const Server_1 = __importDefault(require("./Server"));
const utils = __importStar(require("./utils"));
const view_1 = require("./view");
const SETTINGS = [
    "env",
    "url",
    "port",
    "workers",
    "etag",
    "https",
    "https.cert",
    "https.key",
    "x-powered-by",
    "routing.case-sensitive",
    "routing.ignore-trailing-slash",
    "routing.allow-unsafe-regex",
    "routing.max-param-length",
    "json.escape",
    "json.replacer",
    "json.spaces",
    "jsonp.callback",
    "subdomain.offset",
    "trust.proxy",
    "query.parser",
];
class Foxify extends router_1.default {
    static static = serve_static_1.default;
    _settings = {
        env: process.env.NODE_ENV ?? "development",
        url: process.env.APP_URL ?? "localhost",
        // eslint-disable-next-line @typescript-eslint/no-magic-numbers
        port: process.env.APP_PORT ? +process.env.APP_PORT : 3000,
        workers: process.env.WORKERS ? +process.env.WORKERS : 1,
        https: false,
        "x-powered-by": true,
        "routing.allow-unsafe-regex": false,
        "routing.case-sensitive": true,
        "routing.ignore-trailing-slash": false,
        "routing.max-param-length": 100,
        "json.escape": false,
        "jsonp.callback": "callback",
        "subdomain.offset": 2,
        "query.parser": qs.parse,
        // eslint-disable-next-line no-undefined
        "trust.proxy": undefined,
        // eslint-disable-next-line no-undefined
        etag: undefined,
    };
    _view;
    constructor() {
        super();
        this.set("etag", "weak");
        this.disable("trust.proxy");
    }
    static dotenv(path) {
        utils.assertType("path", "string", path);
        // eslint-disable-next-line @typescript-eslint/no-var-requires,@typescript-eslint/no-require-imports
        require("dotenv").config({ path });
    }
    disable(setting) {
        return this.set(setting, false);
    }
    disabled(setting) {
        return !this.setting(setting);
    }
    enable(setting) {
        return this.set(setting, true);
    }
    enabled(setting) {
        return !this.disabled(setting);
    }
    /**
     * Handle view
     * @param extension view template file extension
     * @param path the directory containing view templates
     * @param handler
     */
    engine(extension, path, handler) {
        this._view = new view_1.Engine(path, extension, handler);
        return this;
    }
    async inject(options) {
        (0, assert_1.default)(this.setting("env") === "test", "Inject only works on the testing environment");
        if (typeof options === "string")
            options = { url: options };
        (0, http_1.requestSettings)(this._settings);
        (0, http_1.responseSettings)({
            ...this._settings,
            view: this._view,
        });
        // TODO: fix typescript issues
        return (0, inject_1.default)(this.lookup.bind(this), {
            ...options,
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-expect-error
            Response: http_1.Response,
            Request: http_1.Request,
        });
    }
    set(setting, value) {
        switch (setting) {
            case "env":
                utils.assertType(setting, "string", value);
                break;
            case "url":
                utils.assertType(setting, "string", value);
                break;
            case "port":
                utils.assertNonNegInt(setting, value);
                break;
            case "workers":
                utils.assertPosInt(setting, value);
                break;
            case "etag":
                if (value == null || typeof value === "function")
                    break;
                // eslint-disable-next-line no-undefined
                if (value === false)
                    value = undefined;
                else if (value === true || value === "weak")
                    value = utils.createETagGenerator(true);
                else if (value === "strong")
                    value = utils.createETagGenerator(false);
                else
                    throw new TypeError(`Unknown value for ${setting} setting: ${value}`);
                break;
            case "https":
                utils.assertType(setting, "boolean", value);
                break;
            case "https.cert":
                utils.assertType(setting, "string", value);
                break;
            case "https.key":
                utils.assertType(setting, "string", value);
                break;
            case "x-powered-by":
                utils.assertType(setting, "boolean", value);
                break;
            case "routing.case-sensitive":
                utils.assertType(setting, "boolean", value);
                break;
            case "routing.ignore-trailing-slash":
                utils.assertType(setting, "boolean", value);
                break;
            case "routing.allow-unsafe-regex":
                utils.assertType(setting, "boolean", value);
                break;
            case "routing.max-param-length":
                utils.assertPosInt(setting, value);
                break;
            case "json.escape":
                utils.assertType(setting, "boolean", value);
                break;
            case "json.replacer":
                utils.assertType(setting, "function", value);
                break;
            case "json.spaces":
                utils.assertNonNegInt(setting, value);
                break;
            case "jsonp.callback":
                utils.assertType(setting, "string", value);
                break;
            case "subdomain.offset":
                utils.assertNonNegInt(setting, value);
                break;
            case "trust.proxy":
                if (value === true) {
                    value = (() => true);
                }
                else if (typeof value === "number") {
                    const num = value;
                    value = ((ip, i) => i < num);
                }
                else if (typeof value !== "function") {
                    if (typeof value === "string")
                        value = value.split(/ *, */);
                    value = proxyAddr.compile(value || []);
                }
                break;
            case "query.parser":
                if (typeof value === "function")
                    break;
                if (value === true || value === "simple")
                    value = qs.parse;
                else if (value === false)
                    value = (() => ({}));
                else if (value === "extended")
                    value = ((str) => qs.parse(str, { allowPrototypes: true }));
                else
                    throw new TypeError(`Unknown value for ${setting} setting: ${value}`);
                break;
            default:
                throw new TypeError(`Expected setting to be one of [${SETTINGS}], got ${setting}`);
        }
        this._settings[setting] = value;
        return this;
    }
    setting(setting) {
        (0, assert_1.default)(SETTINGS.includes(setting), `Expected setting to be one of [${SETTINGS}], got ${setting}`);
        return this._settings[setting];
    }
    start(callback) {
        if (callback != null)
            utils.assertType("callback", "function", callback);
        /* Set node env */
        process.env.NODE_ENV = this.setting("env");
        const server = new Server_1.default({
            ...this._settings,
            view: this._view,
        }, this.lookup.bind(this));
        return server.start(callback);
    }
}
exports.default = Foxify;
//# sourceMappingURL=Foxify.js.map