"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const cluster_1 = __importDefault(require("cluster"));
const http_1 = __importDefault(require("http"));
const https_1 = __importDefault(require("https"));
const http_2 = require("@foxify/http");
class Server {
    _host;
    _listening = false;
    _port;
    _instance;
    constructor(settings, listener) {
        this._host = settings.url;
        this._port = settings.port;
        const isHttps = settings.https;
        const SERVER = isHttps ? https_1.default : http_1.default;
        (0, http_2.requestSettings)(settings);
        (0, http_2.responseSettings)(settings);
        const OPTIONS = {
            IncomingMessage: http_2.Request,
            ServerResponse: http_2.Response,
        };
        if (isHttps) {
            OPTIONS.cert = settings["https.cert"];
            OPTIONS.key = settings["https.key"];
        }
        const workers = settings.workers;
        if (workers > 1) {
            if (cluster_1.default.isPrimary) {
                for (let i = 0; i < workers; i++)
                    cluster_1.default.fork();
                return;
            }
            this._instance = SERVER.createServer(OPTIONS, listener);
            return;
        }
        this._instance = SERVER.createServer(OPTIONS, listener);
    }
    get listening() {
        return this._listening;
    }
    reload(callback) {
        if (this._listening)
            return this.stop(server => server.start(callback));
        return this.start(callback);
    }
    start(callback) {
        this._listening = true;
        const instance = this._instance;
        if (instance) {
            instance.listen(this._port, this._host, callback && (() => callback(this)));
        }
        return this;
    }
    stop(callback) {
        this._listening = false;
        const instance = this._instance;
        if (instance)
            instance.close(callback && (() => callback(this)));
        return this;
    }
}
module.exports = Server;
//# sourceMappingURL=Server.js.map