/// <reference types="node" />
import { Request, Response } from "@foxify/http";
import { InjectResultI, OptionsI as InjectOptionsI } from "@foxify/inject";
import Router from "@foxify/router";
import Server from "./Server";
declare namespace Foxify {
    interface UserSettings {
        env: string;
        etag?: boolean | "strong" | "weak" | ((body: Buffer | string, encoding?: BufferEncoding) => string | undefined) | null;
        https: boolean;
        "https.cert"?: string;
        "https.key"?: string;
        "json.escape": boolean;
        "json.spaces"?: number;
        "jsonp.callback": string;
        port: number;
        "query.parser": boolean | "extended" | "simple" | ((str: string) => Record<string, unknown>);
        "routing.allow-unsafe-regex": boolean;
        "routing.case-sensitive": boolean;
        "routing.ignore-trailing-slash": boolean;
        "routing.max-param-length": number;
        "subdomain.offset": number;
        "trust.proxy": boolean | number | string | ((ip: string, hopIndex: number) => boolean);
        url: string;
        workers: number;
        "x-powered-by": boolean;
        "json.replacer"?(...args: any[]): any;
    }
    interface Settings {
        env: string;
        https: boolean;
        "https.cert"?: string;
        "https.key"?: string;
        "json.escape": boolean;
        "json.spaces"?: number;
        "jsonp.callback": string;
        port: number;
        "routing.allow-unsafe-regex": boolean;
        "routing.case-sensitive": boolean;
        "routing.ignore-trailing-slash": boolean;
        "routing.max-param-length": number;
        "subdomain.offset": number;
        url: string;
        workers: number;
        "x-powered-by": boolean;
        etag(body: Buffer | string, encoding?: BufferEncoding): string | undefined;
        "json.replacer"?(...args: any[]): any;
        "query.parser"(str: string): Record<string, unknown>;
        "trust.proxy"(ip: string, hopIndex: number): boolean;
    }
}
declare class Foxify extends Router {
    static static: any;
    private _settings;
    private _view?;
    constructor();
    static dotenv(path: string): void;
    disable(setting: keyof Foxify.Settings): this;
    disabled(setting: keyof Foxify.Settings): boolean;
    enable(setting: keyof Foxify.Settings): this;
    enabled(setting: keyof Foxify.Settings): boolean;
    /**
     * Handle view
     * @param extension view template file extension
     * @param path the directory containing view templates
     * @param handler
     */
    engine(extension: string, path: string, handler: () => void): this;
    inject(options: InjectOptionsI | string): Promise<InjectResultI<Request, Response>>;
    set<T extends keyof Foxify.UserSettings>(setting: T, value: Foxify.UserSettings[T]): this;
    setting<T extends keyof Foxify.Settings>(setting: T): Foxify.Settings[T];
    start(callback?: Server.Callback): Server;
}
export default Foxify;
//# sourceMappingURL=Foxify.d.ts.map