import cluster from "cluster";
import http from "http";
import https from "https";
import { Request, requestSettings, Response, responseSettings, } from "@foxify/http";
class Server {
    _host;
    _listening = false;
    _port;
    _instance;
    constructor(settings, listener) {
        this._host = settings.url;
        this._port = settings.port;
        const isHttps = settings.https;
        const SERVER = isHttps ? https : http;
        requestSettings(settings);
        responseSettings(settings);
        const OPTIONS = {
            IncomingMessage: Request,
            ServerResponse: Response,
        };
        if (isHttps) {
            OPTIONS.cert = settings["https.cert"];
            OPTIONS.key = settings["https.key"];
        }
        const workers = settings.workers;
        if (workers > 1) {
            if (cluster.isPrimary) {
                for (let i = 0; i < workers; i++)
                    cluster.fork();
                return;
            }
            this._instance = SERVER.createServer(OPTIONS, listener);
            return;
        }
        this._instance = SERVER.createServer(OPTIONS, listener);
    }
    get listening() {
        return this._listening;
    }
    reload(callback) {
        if (this._listening)
            return this.stop(server => server.start(callback));
        return this.start(callback);
    }
    start(callback) {
        this._listening = true;
        const instance = this._instance;
        if (instance) {
            instance.listen(this._port, this._host, callback && (() => callback(this)));
        }
        return this;
    }
    stop(callback) {
        this._listening = false;
        const instance = this._instance;
        if (instance)
            instance.close(callback && (() => callback(this)));
        return this;
    }
}
//# sourceMappingURL=Server.js.map